
# q-expansion of newform 8670.2.a.g, downloaded from the LMFDB on 29 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 8670
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [-1], [1], [4], [0], [2], [0], [-4], [0], [6], [-8], [-2], [6], [-4], [0], [-6], [0], [10], [-4], [0], [-2], [-8], [12], [18], [-2], [18], [-4], [12], [10], [18], [20], [0], [6], [4], [-6], [8], [2], [4], [0], [-18], [24], [-14], [-24], [22], [6], [-8], [-20], [20], [12], [-10], [18], [24], [-2], [-24], [-18], [0], [6], [-16], [-2], [18], [28], [-6], [20], [0], [-2], [-18], [-28], [-26], [-12], [-10], [6], [-24], [28], [26], [4], [0], [-6], [22], [6], [26], [0], [-10], [0], [26], [-8], [12], [6], [26], [-30], [-4], [-36], [24], [28], [24], [4], [-24], [-6], [-18], [20], [10], [28], [18], [-12], [18], [-20], [2], [-12], [30], [-24], [22], [4], [2], [-30], [-44], [32], [30], [4], [24], [-18], [48], [14], [-26], [6], [12], [-44], [-6], [-38], [0], [-28], [-22], [-52], [24], [40], [2], [18], [2], [42], [4], [-30], [54], [4], [-18], [-20], [-12], [38], [-24], [46], [18], [-4], [24], [-2], [54], [-4], [0], [-44], [48], [-16], [6], [26], [-18], [-36], [6], [-4], [24], [-42], [24], [16], [-26], [-2], [54], [48], [14], [-24], [50], [56], [-18], [52], [-42], [-28], [14], [-52], [-48], [-50], [6], [0], [-6], [-2], [-52], [-26], [24], [22], [-12], [-28], [-30], [36], [-6], [-50], [46], [-30], [-24], [-18], [-8], [-22], [-46], [0], [-30], [-40], [-12], [30], [-28], [22], [42], [-44], [60], [48], [-26], [20], [-6], [-24], [66], [-10], [40], [-30], [52], [12], [-58], [-6], [0], [20], [-48], [2], [-20], [-16], [18], [-28], [-24], [-50], [54], [-48], [-48], [-36], [20], [68], [38], [-6], [48], [-68], [-24], [-28], [-24], [-22], [-66], [-48], [-26], [66], [-48], [34], [28], [-42], [2], [-20], [-12], [58], [-26], [18], [44], [-30], [42], [76], [-6], [34], [20], [-46], [-40], [26], [-52], [-36], [38], [-22], [0], [72], [40], [72], [-10], [4], [24], [46], [-42], [8], [66], [30], [-60], [-66], [0], [-26], [-18], [56], [42], [0], [-28], [50], [-78], [-80], [-12], [-68], [46], [-12], [-14], [48], [74], [-72], [-66], [54], [-28], [-72], [26], [-72], [-24], [70], [-30], [-44], [-2], [-54], [-4], [66], [2], [-20], [-44], [-48], [6], [-62], [-66], [88], [60], [52], [-36], [-34], [42], [-74], [20], [74], [78], [78], [-40], [18], [-24], [-38], [-4], [72], [-42], [20], [70], [42], [28], [14], [-66], [-48], [48], [66], [-48], [70], [-42], [0], [0], [92], [-22], [30], [-52], [-26], [-24], [-44], [48], [-78], [-40], [22], [-24], [-48], [-46], [42], [26], [-18], [6], [-92], [42], [-68], [48], [8], [-94], [-92], [-24], [-2], [6], [-48], [-52], [24], [22], [32], [42], [-20], [-54], [-14], [30], [-28], [-54], [-30], [16], [74], [-6], [-4], [48], [-2], [-54], [-60], [-44], [2], [6], [96], [-92], [-78], [96], [-78], [-26], [48], [-72], [-2], [-78], [84], [6], [20], [24], [-22], [24], [4], [-72], [22], [-78], [-26], [-34], [28], [-16], [36], [-42], [86], [72], [-2], [102], [-52], [72], [46], [14], [-52], [-96], [12], [66], [-22], [90], [-10], [-96], [22], [90], [76], [16], [-24], [-38], [-100], [26], [80], [-36], [30], [-4], [76], [-12], [-96], [-98], [-72], [-26], [78], [56], [48], [42], [74], [90], [-2], [-18], [-68], [12], [62], [48], [28], [80], [26], [-48], [-74], [-18], [0], [-14], [28], [90], [-56], [-76], [102], [20], [18], [-44], [-94], [-42], [48], [112], [50], [52], [24], [-24], [-22]]
